#ifndef __IUART_H__
#define __IUART_H__

//-------------------------------------------------------------------------
// user interface
//                                                

                                                    /* Global definitions */
#ifndef __UART_H__
typedef unsigned char  U8;
typedef          char  S8;
typedef unsigned short U16;
typedef          short S16;
typedef unsigned long  U32;
typedef          long  S32;
#endif

extern void IUART_Init();                      // initialize xmt and
                                                     // rcv interrupt handlers

//
// send a line to UART
//
//    *** task may block if tmoMills is non zero ***
//
//    returns 1(ok), 0(fail)
//                                                   
char IUART_Xmt_Line( unsigned char* linePtr,    // ptr to chars to send
                     unsigned char  numChar,    // # of chars to send
                     unsigned int   tmoMills);  // max mills to wait ifque full
//
// recv a line from UART  (null terminated)
//
//    *** task may block if tmoMills is non zero ***
//
//    returns 1(ok), 0(fail) 
//
char IUART_Rcv_Line( unsigned char* linePtr,    // ptr to chars to be rcvd
                     unsigned char  lineSiz,    // # of chars max in buf
                     unsigned int   tmoMills);  // max mills to wait

//
// recv a char from UART  
//
//    *** task may block if tmoMills is non zero ***
//
//    Note: Characters are not available until a line is
//          commited with an end of line character (return). 
//          Otherwise 'bogus' characters would be returned
//          after line edits were made via Control-C etc.
//
//    returns char(ok), 0(timedout) 
//
char IUART_Rcv_Char( unsigned int tmoMills);    // max mills to wait

//
// print hex values
//
extern void IUART_PrintfStr(U8* pBuf);  // print a string using xmt interrupts 
extern void IUART_PrintfU8 (U8  Data);  // print a char as ASCII hex
extern void IUART_PrintfU16(U16 Data);  // print a int  as ASCII hex
extern void IUART_PrintfU32(U32 Data);  // print a long as ASCII hex

extern void IUART_PrintfEndOfLine();    // print end of line \r\n

extern void IUART_sPrintfU8 (U8  Data, U8* pBuf); // save char as ASCII hex in buf
extern void IUART_sPrintfU16(U16 Data, U8* pBuf); // save int  as ASCII hex in buf
extern void IUART_sPrintfU32(U32 Data, U8* pBuf); // save long as ASCII hex in buf


/* Macros */
#define IPRINT(str)     IUART_Xmt_Line(str, strlen(str), WAIT_FOREVER)
#define IREAD(str)      IUART_Rcv_Line(str, sizeof(str), WAIT_FOREVER)
#define IPUTCHAR(str)   IUART_Xmt_Line(str, 1, WAIT_FOREVER)
#define IGETCHAR()      IUART_Rcv_Char(WAIT_FOREVER)
#define IEOL            IUART_PrintfEndOfLine
#define IPRINTU8(data)  IUART_PrintfU8(data)
#define IPRINTU16(data) IUART_PrintfU16(data)
#define IPRINTU32(data) IUART_PrintfU32(data)

//-----------------------------------------------------------------------------
// internal handler stuff
//

/* UART Baud rate calculation */
#define IUART_CPU               4000000      /* 4Mhz */
#define IUART_BAUD_RATE         9600         /* baud rate*/
#define IUART_BAUD_SELECT       (IUART_CPU/(IUART_BAUD_RATE*16l)-1)

#define MAX_UART_RECV_CHARS 200    // size of recv queue
#define MAX_UART_SEND_CHARS 200    // size of xmit queue

#define NO_SIGNAL_PENDING   0xff     // no task wf signal on line rcvd
#define LINE_RCVD_SIGNAL    0x1      // signal send to task a line was rcvd
#define LINE_XMTD_SIGNAL    0x2      // signal send to task a line was xmtd

#define WAIT_FOREVER      0xffff   // wait forever, no timeout

#define DISABLE_RX_TX_INTR  outp((1<<RXEN)|(1<<TXEN), UCR)
#define ENABLE_RX_TX_INTR   outp((1<<RXCIE)|(1<<TXCIE)|(1<<RXEN)|(1<<TXEN), UCR)

#define IUART_ECHO_INPUT  // remove this to disable echo

#define IUART_BS     0x8    // backspace
#define IUART_DEL    0x7f   // delete
#define IUART_ESC    0x1b   // escape
#define IUART_CTL_B  0x2    // control-B
#define IUART_CTL_C  0x3    // control-C
#define IUART_CTL_D  0x4    // control-D
#define IUART_CTL_E  0x5    // control-E
#define IUART_CTL_L  0xc    // control-L
#define IUART_CTL_N  0xe    // control-N
#define IUART_CTL_P  0x10   // control-P
#define IUART_CTL_R  0x12   // control-R
#define IUART_CTL_U  0x15   // control-U

#define IUART_ON_SAME_LINE 0       // rePrint partial input line on same line
#define IUART_ON_NEW_LINE  1       // rePrint partial input line on a new line
#define IUART_MAX_UNDO_CHARS 80    // size of undo line buffer
#define IUART_MAX_LINE_CHARS 80    // max input line size

#define MAX_TRACE_BUF 20    // size of debug trace buffer      


//
// debug print macros
//

//#define DPRINT(string)   UART_PrintfStr(string)
//#define DEOL()           UART_PrintfEndOfLine()
//#define DPRINTU8(data)   UART_PrintfU8(data)
//#define DPRINTU16(data)  UART_PrintfU16(data)
//#define DPRINTU32(data)  UART_PrintfU32(data)

#define DPRINT(string)   
#define DEOL()           
#define DPRINTU8(data)   
#define DPRINTU16(data)  
#define DPRINTU32(data)  


#endif


